#pragma once
//
// If capability is less than 2.0 we need to make our own atomicAdd function.
//
#if defined (__CUDA_ARCH__) && __CUDA_ARCH__ < 200

#if defined (__CUDA_ARCH__) && __CUDA_ARCH__ <= 100
#error Cuda capability less than 1.1 is not supported!
#endif

#define atomicAdd atomicFloatAdd

__device__ void atomicFloatAdd(float* address, float val) 
{ 
	float old = *address, assumed; 
	do { 
		assumed = old; 
		old = __int_as_float(atomicCAS((unsigned int*)address, 
									   __float_as_int(assumed), 
									   __float_as_int(val + assumed))); 
	} while (assumed != old);
}

#else

#ifdef _DOUBLE
#define atomicAdd atomicDoubleAdd

inline __device__ double atomicDoubleAdd(double* address, double val) 
{ 
	double old = *address, assumed; 
	do { 
		assumed = old; 
		old = __longlong_as_double( atomicCAS((unsigned long long int*)address, 
											  __double_as_longlong(assumed), 
											  __double_as_longlong(val + assumed))); 
	} while (assumed != old);
}
#endif

#endif
