//
// Atune - Auto-Tuning of Numerical Applications for GPUs.
//
// - Version 1.0 (April 2011)
//
// - Hans Henrik Brandenborg Sørensen, hhs@imm.dtu.dk
// - GPUlab, DTU Informatics
// - Technical University of Denmark
//
#define m_max 1
#define n_max 177827942
#define m_grid 1
#define n_grid 264
#define m_step (log10(m_max)/m_grid)
#define n_step (log10(n_max)/n_grid)
#define m_idx ((int)(log10(m)/m_step))
#define n_idx ((int)(log10(n)/n_step))

int besttable_scopy[] = { 
8, 1, 14, 1, 1, 1, 1, 1, 1, 1, 11, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 18, 1, 1, 45, 40, 7, 20, 20, 5, 7, 20, 5, 20, 20, 7, 18, 20, 18, 20, 11, 2, 20, 25, 13, 9, 18, 12, 11, 6, 18, 15, 20, 14, 13, 18, 9, 18, 19, 15, 18, 14, 2, 20, 20, 14, 18, 4, 6, 20, 20, 20, 27, 39, 33, 6, 6, 6, 6, 3, 12, 29, 7, 31, 12, 28, 6, 9, 9, 18, 2, 17, 2, 12, 9, 13, 12, 18, 13, 12, 17, 6, 6, 12, 3, 18, 12, 6, 4, 17, 12, 17, 10, 7, 12, 17, 9, 9, 12, 13, 38, 12, 7, 12, 12, 13, 10, 12, 13, 30, 31, 32, 13, 38, 13, 38, 13, 13, 13, 13, 13, 33, 38, 18, 21, 35, 18, 18, 33, 39, 22, 16, 26, 23, 24, 40, 43, 36, 19, 46, 37, 49, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 50, 38, 50, 38, 38, 38, 50, 50, 38, 38, 38, 38, 38, 38, 50, 38, 38, 50, 38, 38, 38, 38, 50, 38, 38, 38, 50, 38, 38, 38, 38, 38, 50, 50, 38, 38, 50, 50, 50, 50, 50, 50, 34, 50, 38, 44, 50, 34, 50, 42, 50, 47, 47, 47, 50, 41, 48, 41, 48, 41, 41, 41, 41, 41, 41, 47, 47, 47, 47, 47, 48, 50, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 
};

int atune_scopy(scopy_arguments)
{
   int bestkernel = besttable_scopy[n_idx];
   switch (bestkernel)
   {
   case 1:
      cudaMemcpy(y_d, x_d, sizeof(x_d[0]) * n, D2D);
      break;
   case 2:
      scopy_v1<1, 16, 1, 1>(x_d, y_d, n);
      break;
   case 3:
      scopy_v1<1, 16, 1, 2>(x_d, y_d, n);
      break;
   case 4:
      scopy_v1<1, 16, 1, 4>(x_d, y_d, n);
      break;
   case 5:
      scopy_v1<1, 16, 1, 8>(x_d, y_d, n);
      break;
   case 6:
      scopy_v1<1, 32, 1, 1>(x_d, y_d, n);
      break;
   case 7:
      scopy_v1<1, 32, 1, 2>(x_d, y_d, n);
      break;
   case 8:
      scopy_v1<1, 32, 1, 16>(x_d, y_d, n);
      break;
   case 9:
      scopy_v1<1, 64, 1, 1>(x_d, y_d, n);
      break;
   case 10:
      scopy_v1<1, 64, 1, 2>(x_d, y_d, n);
      break;
   case 11:
      scopy_v1<1, 64, 1, 16>(x_d, y_d, n);
      break;
   case 12:
      scopy_v1<1, 128, 1, 1>(x_d, y_d, n);
      break;
   case 13:
      scopy_v1<1, 128, 1, 2>(x_d, y_d, n);
      break;
   case 14:
      scopy_v1<1, 128, 1, 8>(x_d, y_d, n);
      break;
   case 15:
      scopy_v1<1, 128, 1, 16>(x_d, y_d, n);
      break;
   case 16:
      scopy_v1<1, 160, 1, 4>(x_d, y_d, n);
      break;
   case 17:
      scopy_v1<1, 256, 1, 1>(x_d, y_d, n);
      break;
   case 18:
      scopy_v1<1, 256, 1, 2>(x_d, y_d, n);
      break;
   case 19:
      scopy_v1<1, 256, 1, 4>(x_d, y_d, n);
      break;
   case 20:
      scopy_v1<1, 256, 1, 8>(x_d, y_d, n);
      break;
   case 21:
      scopy_v1<1, 320, 1, 2>(x_d, y_d, n);
      break;
   case 22:
      scopy_v1<1, 320, 1, 4>(x_d, y_d, n);
      break;
   case 23:
      scopy_v1<1, 416, 1, 4>(x_d, y_d, n);
      break;
   case 24:
      scopy_v1<1, 448, 1, 4>(x_d, y_d, n);
      break;
   case 25:
      scopy_v1<1, 512, 1, 1>(x_d, y_d, n);
      break;
   case 26:
      scopy_v1<1, 512, 1, 4>(x_d, y_d, n);
      break;
   case 27:
      scopy_v1<1, 512, 1, 8>(x_d, y_d, n);
      break;
   case 28:
      scopy_v2<1, 16, 1, 2>(x_d, y_d, n);
      break;
   case 29:
      scopy_v2<1, 32, 1, 2>(x_d, y_d, n);
      break;
   case 30:
      scopy_v2<1, 32, 1, 4>(x_d, y_d, n);
      break;
   case 31:
      scopy_v2<1, 64, 1, 2>(x_d, y_d, n);
      break;
   case 32:
      scopy_v2<1, 128, 1, 2>(x_d, y_d, n);
      break;
   case 33:
      scopy_v2<1, 128, 1, 4>(x_d, y_d, n);
      break;
   case 34:
      scopy_v2<1, 160, 1, 2>(x_d, y_d, n);
      break;
   case 35:
      scopy_v2<1, 160, 1, 4>(x_d, y_d, n);
      break;
   case 36:
      scopy_v2<1, 192, 1, 8>(x_d, y_d, n);
      break;
   case 37:
      scopy_v2<1, 224, 1, 8>(x_d, y_d, n);
      break;
   case 38:
      scopy_v2<1, 256, 1, 2>(x_d, y_d, n);
      break;
   case 39:
      scopy_v2<1, 256, 1, 4>(x_d, y_d, n);
      break;
   case 40:
      scopy_v2<1, 256, 1, 8>(x_d, y_d, n);
      break;
   case 41:
      scopy_v2<1, 320, 1, 2>(x_d, y_d, n);
      break;
   case 42:
      scopy_v2<1, 352, 1, 2>(x_d, y_d, n);
      break;
   case 43:
      scopy_v2<1, 352, 1, 8>(x_d, y_d, n);
      break;
   case 44:
      scopy_v2<1, 384, 1, 2>(x_d, y_d, n);
      break;
   case 45:
      scopy_v2<1, 384, 1, 8>(x_d, y_d, n);
      break;
   case 46:
      scopy_v2<1, 416, 1, 8>(x_d, y_d, n);
      break;
   case 47:
      scopy_v2<1, 448, 1, 2>(x_d, y_d, n);
      break;
   case 48:
      scopy_v2<1, 480, 1, 2>(x_d, y_d, n);
      break;
   case 49:
      scopy_v2<1, 480, 1, 8>(x_d, y_d, n);
      break;
   case 50:
      scopy_v2<1, 512, 1, 2>(x_d, y_d, n);
      break;
   case 51:
      scopy_v2<1, 1024, 1, 4>(x_d, y_d, n);
      break;
   default:
      printf("\nscopy.h : bestkernelSwitch : problem size not supported (n = %d, bestkernel = %d).\n", n, bestkernel);
      exit(EXIT_FAILURE);
   }
   return bestkernel;
}
