//
// Atune - Auto-Tuning of Numerical Applications for GPUs.
//
// - Version 1.0 (April 2011)
//
// - Hans Henrik Brandenborg Sørensen, hhs@imm.dtu.dk
// - GPUlab, DTU Informatics
// - Technical University of Denmark
//
#define m_max 1
#define n_max 177827942
#define m_grid 1
#define n_grid 264
#define m_step (log10(m_max)/m_grid)
#define n_step (log10(n_max)/n_grid)
#define m_idx ((int)(log10(m)/m_step))
#define n_idx ((int)(log10(n)/n_step))

int besttable_sdot[] = { 
4, 1, 2, 7, 9, 8, 5, 9, 25, 9, 9, 7, 22, 5, 9, 5, 9, 5, 53, 28, 42, 34, 19, 2, 9, 14, 16, 6, 19, 15, 6, 5, 7, 8, 10, 6, 19, 7, 6, 3, 16, 24, 17, 1, 13, 17, 10, 16, 2, 24, 10, 25, 19, 3, 37, 1, 10, 2, 10, 3, 16, 1, 1, 10, 2, 10, 10, 10, 12, 3, 1, 1, 19, 3, 1, 10, 10, 11, 2, 10, 3, 10, 10, 2, 10, 10, 10, 20, 10, 10, 11, 10, 2, 10, 10, 3, 11, 32, 11, 11, 18, 3, 18, 18, 31, 19, 21, 4, 30, 4, 12, 11, 12, 12, 12, 30, 31, 21, 32, 31, 13, 21, 47, 31, 47, 31, 13, 32, 20, 32, 13, 32, 47, 21, 47, 33, 33, 47, 26, 47, 33, 45, 49, 48, 38, 33, 43, 44, 38, 43, 34, 46, 38, 34, 27, 43, 44, 44, 44, 34, 58, 43, 60, 54, 43, 49, 39, 57, 57, 59, 44, 52, 51, 51, 28, 50, 35, 28, 35, 23, 35, 40, 35, 25, 29, 41, 23, 23, 27, 27, 36, 23, 23, 27, 23, 27, 55, 23, 27, 22, 27, 23, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 56, 56, 56, 56, 56, 27, 56, 56, 56, 56, 56, 23, 23, 56, 56, 56, 56, 
};

int atune_sdot(sdot_arguments)
{
   int bestkernel = besttable_sdot[n_idx];
   switch (bestkernel)
   {
   case 1:
      sdot_v1<1, 16, 1, 1>(x_d, y_d, z_d, n);
      break;
   case 2:
      sdot_v1<1, 16, 1, 2>(x_d, y_d, z_d, n);
      break;
   case 3:
      sdot_v1<1, 16, 1, 4>(x_d, y_d, z_d, n);
      break;
   case 4:
      sdot_v1<1, 16, 1, 8>(x_d, y_d, z_d, n);
      break;
   case 5:
      sdot_v1<1, 16, 1, 16>(x_d, y_d, z_d, n);
      break;
   case 6:
      sdot_v1<1, 16, 1, 32>(x_d, y_d, z_d, n);
      break;
   case 7:
      sdot_v1<1, 16, 1, 64>(x_d, y_d, z_d, n);
      break;
   case 8:
      sdot_v1<1, 16, 1, 128>(x_d, y_d, z_d, n);
      break;
   case 9:
      sdot_v1<1, 16, 1, 256>(x_d, y_d, z_d, n);
      break;
   case 10:
      sdot_v1<1, 32, 1, 1>(x_d, y_d, z_d, n);
      break;
   case 11:
      sdot_v1<1, 32, 1, 2>(x_d, y_d, z_d, n);
      break;
   case 12:
      sdot_v1<1, 32, 1, 4>(x_d, y_d, z_d, n);
      break;
   case 13:
      sdot_v1<1, 32, 1, 8>(x_d, y_d, z_d, n);
      break;
   case 14:
      sdot_v1<1, 32, 1, 16>(x_d, y_d, z_d, n);
      break;
   case 15:
      sdot_v1<1, 32, 1, 32>(x_d, y_d, z_d, n);
      break;
   case 16:
      sdot_v1<1, 32, 1, 64>(x_d, y_d, z_d, n);
      break;
   case 17:
      sdot_v1<1, 32, 1, 128>(x_d, y_d, z_d, n);
      break;
   case 18:
      sdot_v1<1, 64, 1, 1>(x_d, y_d, z_d, n);
      break;
   case 19:
      sdot_v1<1, 64, 1, 2>(x_d, y_d, z_d, n);
      break;
   case 20:
      sdot_v1<1, 64, 1, 4>(x_d, y_d, z_d, n);
      break;
   case 21:
      sdot_v1<1, 64, 1, 8>(x_d, y_d, z_d, n);
      break;
   case 22:
      sdot_v1<1, 64, 1, 16>(x_d, y_d, z_d, n);
      break;
   case 23:
      sdot_v1<1, 64, 1, 32>(x_d, y_d, z_d, n);
      break;
   case 24:
      sdot_v1<1, 64, 1, 64>(x_d, y_d, z_d, n);
      break;
   case 25:
      sdot_v1<1, 64, 1, 128>(x_d, y_d, z_d, n);
      break;
   case 26:
      sdot_v1<1, 96, 1, 8>(x_d, y_d, z_d, n);
      break;
   case 27:
      sdot_v1<1, 96, 1, 16>(x_d, y_d, z_d, n);
      break;
   case 28:
      sdot_v1<1, 96, 1, 64>(x_d, y_d, z_d, n);
      break;
   case 29:
      sdot_v1<1, 96, 1, 128>(x_d, y_d, z_d, n);
      break;
   case 30:
      sdot_v1<1, 128, 1, 1>(x_d, y_d, z_d, n);
      break;
   case 31:
      sdot_v1<1, 128, 1, 2>(x_d, y_d, z_d, n);
      break;
   case 32:
      sdot_v1<1, 128, 1, 4>(x_d, y_d, z_d, n);
      break;
   case 33:
      sdot_v1<1, 128, 1, 8>(x_d, y_d, z_d, n);
      break;
   case 34:
      sdot_v1<1, 128, 1, 16>(x_d, y_d, z_d, n);
      break;
   case 35:
      sdot_v1<1, 128, 1, 32>(x_d, y_d, z_d, n);
      break;
   case 36:
      sdot_v1<1, 128, 1, 64>(x_d, y_d, z_d, n);
      break;
   case 37:
      sdot_v1<1, 128, 1, 256>(x_d, y_d, z_d, n);
      break;
   case 38:
      sdot_v1<1, 160, 1, 8>(x_d, y_d, z_d, n);
      break;
   case 39:
      sdot_v1<1, 160, 1, 16>(x_d, y_d, z_d, n);
      break;
   case 40:
      sdot_v1<1, 160, 1, 32>(x_d, y_d, z_d, n);
      break;
   case 41:
      sdot_v1<1, 160, 1, 128>(x_d, y_d, z_d, n);
      break;
   case 42:
      sdot_v1<1, 160, 1, 256>(x_d, y_d, z_d, n);
      break;
   case 43:
      sdot_v1<1, 192, 1, 8>(x_d, y_d, z_d, n);
      break;
   case 44:
      sdot_v1<1, 192, 1, 16>(x_d, y_d, z_d, n);
      break;
   case 45:
      sdot_v1<1, 224, 1, 4>(x_d, y_d, z_d, n);
      break;
   case 46:
      sdot_v1<1, 224, 1, 16>(x_d, y_d, z_d, n);
      break;
   case 47:
      sdot_v1<1, 256, 1, 2>(x_d, y_d, z_d, n);
      break;
   case 48:
      sdot_v1<1, 256, 1, 4>(x_d, y_d, z_d, n);
      break;
   case 49:
      sdot_v1<1, 256, 1, 8>(x_d, y_d, z_d, n);
      break;
   case 50:
      sdot_v1<1, 256, 1, 16>(x_d, y_d, z_d, n);
      break;
   case 51:
      sdot_v1<1, 288, 1, 64>(x_d, y_d, z_d, n);
      break;
   case 52:
      sdot_v1<1, 320, 1, 16>(x_d, y_d, z_d, n);
      break;
   case 53:
      sdot_v1<1, 352, 1, 1>(x_d, y_d, z_d, n);
      break;
   case 54:
      sdot_v1<1, 352, 1, 8>(x_d, y_d, z_d, n);
      break;
   case 55:
      sdot_v1<1, 352, 1, 256>(x_d, y_d, z_d, n);
      break;
   case 56:
      sdot_v1<1, 384, 1, 8>(x_d, y_d, z_d, n);
      break;
   case 57:
      sdot_v1<1, 384, 1, 16>(x_d, y_d, z_d, n);
      break;
   case 58:
      sdot_v1<1, 416, 1, 8>(x_d, y_d, z_d, n);
      break;
   case 59:
      sdot_v1<1, 416, 1, 16>(x_d, y_d, z_d, n);
      break;
   case 60:
      sdot_v1<1, 480, 1, 8>(x_d, y_d, z_d, n);
      break;
   default:
      printf("\nsdot.h : bestkernelSwitch : problem size not supported (n = %d, bestkernel = %d).\n", n, bestkernel);
      exit(EXIT_FAILURE);
   }
   return bestkernel;
}
