//
// Atune - Auto-Tuning of Numerical Applications for GPUs.
//
// - Version 1.0 (April 2011)
//
// - Hans Henrik Brandenborg Sørensen, hhs@imm.dtu.dk
// - GPUlab, DTU Informatics
// - Technical University of Denmark
//
#define m_max 1
#define n_max 177827942
#define m_grid 1
#define n_grid 264
#define m_step (log10(m_max)/m_grid)
#define n_step (log10(n_max)/n_grid)
#define m_idx ((int)(log10(m)/m_step))
#define n_idx ((int)(log10(n)/n_step))

int besttable_smset[] = { 
1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 25, 1, 1, 24, 24, 17, 1, 1, 1, 1, 1, 1, 1, 17, 9, 1, 37, 3, 24, 24, 1, 1, 1, 8, 8, 1, 24, 2, 14, 9, 9, 1, 13, 7, 1, 13, 1, 38, 26, 1, 8, 39, 6, 11, 11, 1, 3, 38, 1, 1, 4, 1, 8, 25, 1, 1, 13, 1, 13, 5, 1, 1, 15, 1, 7, 3, 1, 1, 15, 15, 24, 14, 7, 10, 10, 1, 8, 10, 36, 11, 24, 4, 4, 5, 8, 14, 14, 24, 8, 24, 1, 1, 23, 36, 26, 24, 15, 1, 1, 24, 1, 24, 15, 1, 9, 38, 25, 1, 24, 24, 38, 25, 15, 25, 24, 21, 25, 18, 19, 39, 39, 20, 21, 31, 28, 19, 22, 32, 26, 13, 22, 21, 16, 16, 34, 19, 18, 21, 9, 13, 13, 13, 9, 16, 13, 12, 9, 9, 13, 9, 9, 13, 13, 9, 9, 9, 13, 9, 12, 9, 9, 9, 12, 9, 12, 9, 9, 9, 15, 9, 15, 24, 12, 9, 15, 9, 9, 13, 9, 13, 13, 9, 13, 12, 9, 9, 9, 9, 9, 9, 9, 12, 12, 9, 9, 12, 12, 12, 15, 12, 37, 24, 24, 24, 24, 37, 13, 24, 27, 27, 37, 37, 37, 30, 37, 37, 37, 37, 37, 33, 33, 29, 33, 35, 33, 35, 35, 38, 38, 40, 40, 40, 40, 
};

int atune_smset(smset_arguments)
{
   int bestkernel = besttable_smset[n_idx];
   switch (bestkernel)
   {
   case 1:
      cudaMemset(y_d, val, sizeof(y_d[0]) * n);
      break;
   case 2:
      smset_v1<1, 16, 1, 1>(val, y_d, n);
      break;
   case 3:
      smset_v1<1, 16, 1, 2>(val, y_d, n);
      break;
   case 4:
      smset_v1<1, 16, 1, 4>(val, y_d, n);
      break;
   case 5:
      smset_v1<1, 16, 1, 8>(val, y_d, n);
      break;
   case 6:
      smset_v1<1, 32, 1, 1>(val, y_d, n);
      break;
   case 7:
      smset_v1<1, 32, 1, 2>(val, y_d, n);
      break;
   case 8:
      smset_v1<1, 32, 1, 4>(val, y_d, n);
      break;
   case 9:
      smset_v1<1, 32, 1, 8>(val, y_d, n);
      break;
   case 10:
      smset_v1<1, 64, 1, 1>(val, y_d, n);
      break;
   case 11:
      smset_v1<1, 64, 1, 2>(val, y_d, n);
      break;
   case 12:
      smset_v1<1, 64, 1, 4>(val, y_d, n);
      break;
   case 13:
      smset_v1<1, 64, 1, 8>(val, y_d, n);
      break;
   case 14:
      smset_v1<1, 128, 1, 1>(val, y_d, n);
      break;
   case 15:
      smset_v1<1, 128, 1, 2>(val, y_d, n);
      break;
   case 16:
      smset_v1<1, 128, 1, 4>(val, y_d, n);
      break;
   case 17:
      smset_v1<1, 128, 1, 8>(val, y_d, n);
      break;
   case 18:
      smset_v1<1, 160, 1, 4>(val, y_d, n);
      break;
   case 19:
      smset_v1<1, 160, 1, 8>(val, y_d, n);
      break;
   case 20:
      smset_v1<1, 192, 1, 4>(val, y_d, n);
      break;
   case 21:
      smset_v1<1, 192, 1, 8>(val, y_d, n);
      break;
   case 22:
      smset_v1<1, 224, 1, 8>(val, y_d, n);
      break;
   case 23:
      smset_v1<1, 256, 1, 1>(val, y_d, n);
      break;
   case 24:
      smset_v1<1, 256, 1, 2>(val, y_d, n);
      break;
   case 25:
      smset_v1<1, 256, 1, 4>(val, y_d, n);
      break;
   case 26:
      smset_v1<1, 256, 1, 8>(val, y_d, n);
      break;
   case 27:
      smset_v1<1, 288, 1, 2>(val, y_d, n);
      break;
   case 28:
      smset_v1<1, 288, 1, 8>(val, y_d, n);
      break;
   case 29:
      smset_v1<1, 320, 1, 4>(val, y_d, n);
      break;
   case 30:
      smset_v1<1, 352, 1, 2>(val, y_d, n);
      break;
   case 31:
      smset_v1<1, 384, 1, 4>(val, y_d, n);
      break;
   case 32:
      smset_v1<1, 384, 1, 8>(val, y_d, n);
      break;
   case 33:
      smset_v1<1, 416, 1, 4>(val, y_d, n);
      break;
   case 34:
      smset_v1<1, 416, 1, 8>(val, y_d, n);
      break;
   case 35:
      smset_v1<1, 448, 1, 4>(val, y_d, n);
      break;
   case 36:
      smset_v1<1, 512, 1, 1>(val, y_d, n);
      break;
   case 37:
      smset_v1<1, 512, 1, 2>(val, y_d, n);
      break;
   case 38:
      smset_v1<1, 512, 1, 4>(val, y_d, n);
      break;
   case 39:
      smset_v1<1, 512, 1, 8>(val, y_d, n);
      break;
   case 40:
      smset_v1<1, 1024, 1, 8>(val, y_d, n);
      break;
   default:
      printf("\nsmset.h : bestkernelSwitch : problem size not supported (n = %d, bestkernel = %d).\n", n, bestkernel);
      exit(EXIT_FAILURE);
   }
   return bestkernel;
}
