//
// Atune - Auto-Tuning of Numerical Applications for GPUs.
//
// - Version 1.0 (April 2011)
//
// - Hans Henrik Brandenborg Sørensen, hhs@imm.dtu.dk
// - GPUlab, DTU Informatics
// - Technical University of Denmark
//
#define m_max 1
#define n_max 177827942
#define m_grid 1
#define n_grid 264
#define m_step (log10(m_max)/m_grid)
#define n_step (log10(n_max)/n_grid)
#define m_idx ((int)(log10(m)/m_step))
#define n_idx ((int)(log10(n)/n_step))

int besttable_srot[] = { 
1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 51, 33, 33, 8, 17, 31, 16, 16, 33, 8, 4, 7, 25, 33, 32, 27, 27, 2, 7, 33, 28, 17, 16, 30, 28, 16, 28, 17, 7, 30, 33, 33, 29, 27, 17, 30, 33, 30, 27, 33, 33, 26, 13, 25, 31, 28, 3, 25, 26, 32, 30, 30, 25, 28, 2, 56, 12, 53, 25, 25, 25, 25, 25, 57, 25, 5, 5, 25, 25, 3, 5, 25, 25, 25, 25, 19, 3, 26, 25, 25, 11, 25, 26, 34, 11, 25, 25, 35, 18, 46, 52, 3, 5, 52, 52, 45, 43, 40, 34, 9, 43, 25, 36, 36, 58, 12, 18, 18, 22, 52, 12, 52, 18, 12, 25, 53, 12, 22, 10, 22, 13, 13, 6, 53, 38, 20, 27, 27, 44, 47, 38, 20, 13, 20, 24, 41, 28, 48, 50, 54, 14, 28, 54, 28, 28, 28, 29, 55, 29, 40, 15, 55, 29, 35, 11, 53, 12, 53, 52, 43, 49, 21, 12, 52, 53, 12, 18, 22, 12, 60, 37, 11, 34, 39, 23, 25, 25, 25, 25, 42, 12, 42, 36, 36, 36, 52, 21, 52, 39, 11, 25, 11, 52, 25, 11, 18, 34, 49, 18, 11, 25, 49, 18, 46, 39, 23, 52, 36, 52, 49, 36, 46, 46, 39, 46, 46, 46, 49, 52, 59, 59, 59, 59, 59, 59, 59, 59, 59, 59, 59, 59, 59, 59, 59, 59, 59, 59, 59, 59, 60, 60, 60, 60, 
};

int atune_srot(srot_arguments)
{
   int bestkernel = besttable_srot[n_idx];
   switch (bestkernel)
   {
   case 1:
      cublasSrot(n, x_d, 1, y_d, 1, c, s);
      break;
   case 2:
      srot_v1<1, 16, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 3:
      srot_v1<1, 32, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 4:
      srot_v1<1, 32, 1, 8>(x_d, y_d, c, s, n);
      break;
   case 5:
      srot_v1<1, 64, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 6:
      srot_v1<1, 64, 1, 4>(x_d, y_d, c, s, n);
      break;
   case 7:
      srot_v1<1, 64, 1, 32>(x_d, y_d, c, s, n);
      break;
   case 8:
      srot_v1<1, 64, 1, 64>(x_d, y_d, c, s, n);
      break;
   case 9:
      srot_v1<1, 96, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 10:
      srot_v1<1, 96, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 11:
      srot_v1<1, 128, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 12:
      srot_v1<1, 128, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 13:
      srot_v1<1, 128, 1, 4>(x_d, y_d, c, s, n);
      break;
   case 14:
      srot_v1<1, 128, 1, 8>(x_d, y_d, c, s, n);
      break;
   case 15:
      srot_v1<1, 128, 1, 16>(x_d, y_d, c, s, n);
      break;
   case 16:
      srot_v1<1, 128, 1, 128>(x_d, y_d, c, s, n);
      break;
   case 17:
      srot_v1<1, 128, 1, 256>(x_d, y_d, c, s, n);
      break;
   case 18:
      srot_v1<1, 160, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 19:
      srot_v1<1, 160, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 20:
      srot_v1<1, 160, 1, 4>(x_d, y_d, c, s, n);
      break;
   case 21:
      srot_v1<1, 192, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 22:
      srot_v1<1, 192, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 23:
      srot_v1<1, 224, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 24:
      srot_v1<1, 224, 1, 8>(x_d, y_d, c, s, n);
      break;
   case 25:
      srot_v1<1, 256, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 26:
      srot_v1<1, 256, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 27:
      srot_v1<1, 256, 1, 4>(x_d, y_d, c, s, n);
      break;
   case 28:
      srot_v1<1, 256, 1, 8>(x_d, y_d, c, s, n);
      break;
   case 29:
      srot_v1<1, 256, 1, 16>(x_d, y_d, c, s, n);
      break;
   case 30:
      srot_v1<1, 256, 1, 32>(x_d, y_d, c, s, n);
      break;
   case 31:
      srot_v1<1, 256, 1, 64>(x_d, y_d, c, s, n);
      break;
   case 32:
      srot_v1<1, 256, 1, 128>(x_d, y_d, c, s, n);
      break;
   case 33:
      srot_v1<1, 256, 1, 256>(x_d, y_d, c, s, n);
      break;
   case 34:
      srot_v1<1, 288, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 35:
      srot_v1<1, 288, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 36:
      srot_v1<1, 320, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 37:
      srot_v1<1, 320, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 38:
      srot_v1<1, 320, 1, 4>(x_d, y_d, c, s, n);
      break;
   case 39:
      srot_v1<1, 352, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 40:
      srot_v1<1, 352, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 41:
      srot_v1<1, 352, 1, 8>(x_d, y_d, c, s, n);
      break;
   case 42:
      srot_v1<1, 384, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 43:
      srot_v1<1, 384, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 44:
      srot_v1<1, 384, 1, 8>(x_d, y_d, c, s, n);
      break;
   case 45:
      srot_v1<1, 416, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 46:
      srot_v1<1, 448, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 47:
      srot_v1<1, 448, 1, 4>(x_d, y_d, c, s, n);
      break;
   case 48:
      srot_v1<1, 448, 1, 8>(x_d, y_d, c, s, n);
      break;
   case 49:
      srot_v1<1, 480, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 50:
      srot_v1<1, 480, 1, 8>(x_d, y_d, c, s, n);
      break;
   case 51:
      srot_v1<1, 480, 1, 32>(x_d, y_d, c, s, n);
      break;
   case 52:
      srot_v1<1, 512, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 53:
      srot_v1<1, 512, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 54:
      srot_v1<1, 512, 1, 8>(x_d, y_d, c, s, n);
      break;
   case 55:
      srot_v1<1, 512, 1, 16>(x_d, y_d, c, s, n);
      break;
   case 56:
      srot_v1<1, 512, 1, 128>(x_d, y_d, c, s, n);
      break;
   case 57:
      srot_v1<1, 512, 1, 256>(x_d, y_d, c, s, n);
      break;
   case 58:
      srot_v1<1, 1024, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 59:
      srot_v1<1, 1024, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 60:
      srot_v1<1, 1024, 1, 4>(x_d, y_d, c, s, n);
      break;
   default:
      printf("\nsrot.h : bestkernelSwitch : problem size not supported (n = %d, bestkernel = %d).\n", n, bestkernel);
      exit(EXIT_FAILURE);
   }
   return bestkernel;
}
