//
// Atune - Auto-Tuning of Numerical Applications for GPUs.
//
// - Version 1.0 (April 2011)
//
// - Hans Henrik Brandenborg Sørensen, hhs@imm.dtu.dk
// - GPUlab, DTU Informatics
// - Technical University of Denmark
//
#define m_max 1
#define n_max 177827942
#define m_grid 1
#define n_grid 264
#define m_step (log10(m_max)/m_grid)
#define n_step (log10(n_max)/n_grid)
#define m_idx ((int)(log10(m)/m_step))
#define n_idx ((int)(log10(n)/n_step))

int besttable_sscal[] = { 
1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 7, 7, 31, 7, 31, 11, 6, 31, 8, 33, 4, 31, 4, 32, 31, 7, 6, 5, 17, 29, 5, 31, 31, 2, 9, 6, 2, 33, 5, 3, 32, 29, 33, 31, 2, 2, 3, 33, 2, 19, 6, 31, 30, 8, 32, 6, 6, 33, 29, 8, 33, 3, 2, 30, 31, 2, 2, 5, 6, 29, 1, 6, 3, 5, 5, 8, 30, 8, 29, 5, 2, 8, 6, 8, 30, 2, 5, 5, 70, 60, 50, 61, 67, 3, 46, 35, 29, 72, 66, 50, 45, 39, 71, 54, 17, 34, 72, 53, 42, 62, 67, 41, 53, 15, 67, 12, 30, 7, 20, 61, 29, 30, 47, 16, 18, 1, 10, 16, 16, 16, 18, 16, 54, 10, 36, 18, 31, 44, 32, 48, 23, 56, 25, 22, 42, 55, 10, 55, 48, 51, 57, 57, 30, 30, 30, 17, 21, 17, 52, 63, 59, 43, 26, 43, 38, 54, 14, 56, 18, 36, 30, 30, 67, 65, 54, 37, 28, 73, 62, 54, 54, 54, 73, 74, 21, 21, 68, 21, 58, 54, 16, 16, 21, 44, 37, 40, 67, 16, 27, 13, 54, 16, 72, 21, 27, 21, 54, 21, 49, 13, 16, 36, 54, 21, 69, 16, 16, 16, 69, 64, 69, 67, 69, 24, 69, 69, 69, 69, 72, 69, 72, 54, 72, 72, 72, 72, 72, 72, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 
};

int atune_sscal(sscal_arguments)
{
   int bestkernel = besttable_sscal[n_idx];
   switch (bestkernel)
   {
   case 1:
      cublasSscal(n, alpha, x_d, 1);
      break;
   case 2:
      sscal_v1<1, 16, 1, 1>(alpha, x_d, n);
      break;
   case 3:
      sscal_v1<1, 16, 1, 2>(alpha, x_d, n);
      break;
   case 4:
      sscal_v1<1, 16, 1, 4>(alpha, x_d, n);
      break;
   case 5:
      sscal_v1<1, 32, 1, 1>(alpha, x_d, n);
      break;
   case 6:
      sscal_v1<1, 32, 1, 2>(alpha, x_d, n);
      break;
   case 7:
      sscal_v1<1, 32, 1, 4>(alpha, x_d, n);
      break;
   case 8:
      sscal_v1<1, 64, 1, 1>(alpha, x_d, n);
      break;
   case 9:
      sscal_v1<1, 64, 1, 2>(alpha, x_d, n);
      break;
   case 10:
      sscal_v1<1, 64, 1, 8>(alpha, x_d, n);
      break;
   case 11:
      sscal_v1<1, 64, 1, 16>(alpha, x_d, n);
      break;
   case 12:
      sscal_v1<1, 96, 1, 2>(alpha, x_d, n);
      break;
   case 13:
      sscal_v1<1, 96, 1, 4>(alpha, x_d, n);
      break;
   case 14:
      sscal_v1<1, 96, 1, 16>(alpha, x_d, n);
      break;
   case 15:
      sscal_v1<1, 128, 1, 1>(alpha, x_d, n);
      break;
   case 16:
      sscal_v1<1, 128, 1, 2>(alpha, x_d, n);
      break;
   case 17:
      sscal_v1<1, 128, 1, 4>(alpha, x_d, n);
      break;
   case 18:
      sscal_v1<1, 128, 1, 8>(alpha, x_d, n);
      break;
   case 19:
      sscal_v1<1, 128, 1, 16>(alpha, x_d, n);
      break;
   case 20:
      sscal_v1<1, 160, 1, 1>(alpha, x_d, n);
      break;
   case 21:
      sscal_v1<1, 160, 1, 2>(alpha, x_d, n);
      break;
   case 22:
      sscal_v1<1, 160, 1, 4>(alpha, x_d, n);
      break;
   case 23:
      sscal_v1<1, 160, 1, 8>(alpha, x_d, n);
      break;
   case 24:
      sscal_v1<1, 192, 1, 2>(alpha, x_d, n);
      break;
   case 25:
      sscal_v1<1, 192, 1, 8>(alpha, x_d, n);
      break;
   case 26:
      sscal_v1<1, 192, 1, 16>(alpha, x_d, n);
      break;
   case 27:
      sscal_v1<1, 224, 1, 2>(alpha, x_d, n);
      break;
   case 28:
      sscal_v1<1, 224, 1, 8>(alpha, x_d, n);
      break;
   case 29:
      sscal_v1<1, 256, 1, 1>(alpha, x_d, n);
      break;
   case 30:
      sscal_v1<1, 256, 1, 2>(alpha, x_d, n);
      break;
   case 31:
      sscal_v1<1, 256, 1, 4>(alpha, x_d, n);
      break;
   case 32:
      sscal_v1<1, 256, 1, 8>(alpha, x_d, n);
      break;
   case 33:
      sscal_v1<1, 256, 1, 16>(alpha, x_d, n);
      break;
   case 34:
      sscal_v1<1, 288, 1, 4>(alpha, x_d, n);
      break;
   case 35:
      sscal_v1<1, 320, 1, 1>(alpha, x_d, n);
      break;
   case 36:
      sscal_v1<1, 320, 1, 2>(alpha, x_d, n);
      break;
   case 37:
      sscal_v1<1, 320, 1, 4>(alpha, x_d, n);
      break;
   case 38:
      sscal_v1<1, 320, 1, 16>(alpha, x_d, n);
      break;
   case 39:
      sscal_v1<1, 352, 1, 1>(alpha, x_d, n);
      break;
   case 40:
      sscal_v1<1, 352, 1, 2>(alpha, x_d, n);
      break;
   case 41:
      sscal_v1<1, 352, 1, 4>(alpha, x_d, n);
      break;
   case 42:
      sscal_v1<1, 352, 1, 8>(alpha, x_d, n);
      break;
   case 43:
      sscal_v1<1, 352, 1, 16>(alpha, x_d, n);
      break;
   case 44:
      sscal_v1<1, 384, 1, 2>(alpha, x_d, n);
      break;
   case 45:
      sscal_v1<1, 384, 1, 4>(alpha, x_d, n);
      break;
   case 46:
      sscal_v1<1, 416, 1, 1>(alpha, x_d, n);
      break;
   case 47:
      sscal_v1<1, 416, 1, 2>(alpha, x_d, n);
      break;
   case 48:
      sscal_v1<1, 416, 1, 8>(alpha, x_d, n);
      break;
   case 49:
      sscal_v1<1, 448, 1, 2>(alpha, x_d, n);
      break;
   case 50:
      sscal_v1<1, 480, 1, 1>(alpha, x_d, n);
      break;
   case 51:
      sscal_v1<1, 480, 1, 8>(alpha, x_d, n);
      break;
   case 52:
      sscal_v1<1, 480, 1, 16>(alpha, x_d, n);
      break;
   case 53:
      sscal_v1<1, 512, 1, 1>(alpha, x_d, n);
      break;
   case 54:
      sscal_v1<1, 512, 1, 2>(alpha, x_d, n);
      break;
   case 55:
      sscal_v1<1, 512, 1, 4>(alpha, x_d, n);
      break;
   case 56:
      sscal_v1<1, 512, 1, 8>(alpha, x_d, n);
      break;
   case 57:
      sscal_v1<1, 512, 1, 16>(alpha, x_d, n);
      break;
   case 58:
      sscal_v1<1, 1024, 1, 4>(alpha, x_d, n);
      break;
   case 59:
      sscal_v1<1, 1024, 1, 16>(alpha, x_d, n);
      break;
   case 60:
      sscal_v2<1, 32, 1, 2>(alpha, x_d, n);
      break;
   case 61:
      sscal_v2<1, 128, 1, 2>(alpha, x_d, n);
      break;
   case 62:
      sscal_v2<1, 128, 1, 4>(alpha, x_d, n);
      break;
   case 63:
      sscal_v2<1, 128, 1, 16>(alpha, x_d, n);
      break;
   case 64:
      sscal_v2<1, 160, 1, 2>(alpha, x_d, n);
      break;
   case 65:
      sscal_v2<1, 192, 1, 8>(alpha, x_d, n);
      break;
   case 66:
      sscal_v2<1, 224, 1, 4>(alpha, x_d, n);
      break;
   case 67:
      sscal_v2<1, 256, 1, 2>(alpha, x_d, n);
      break;
   case 68:
      sscal_v2<1, 256, 1, 4>(alpha, x_d, n);
      break;
   case 69:
      sscal_v2<1, 288, 1, 2>(alpha, x_d, n);
      break;
   case 70:
      sscal_v2<1, 320, 1, 2>(alpha, x_d, n);
      break;
   case 71:
      sscal_v2<1, 384, 1, 2>(alpha, x_d, n);
      break;
   case 72:
      sscal_v2<1, 512, 1, 2>(alpha, x_d, n);
      break;
   case 73:
      sscal_v2<1, 512, 1, 4>(alpha, x_d, n);
      break;
   case 74:
      sscal_v2<1, 1024, 1, 8>(alpha, x_d, n);
      break;
   default:
      printf("\nsscal.h : bestkernelSwitch : problem size not supported (n = %d, bestkernel = %d).\n", n, bestkernel);
      exit(EXIT_FAILURE);
   }
   return bestkernel;
}
