#pragma once

#define call_kernel(kernel, BLOCKSIZE_m, WORKSIZE_m, WORKSIZE_n) \
	kernel<BLOCKSIZE_m, WORKSIZE_m, WORKSIZE_n><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile);

#define call_kernel_all(kernel, BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, WORKSIZE_n) \
	kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, WORKSIZE_n><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile);

#define call_kernel_sm(kernel, BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, WORKSIZE_n) \
	kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, WORKSIZE_n><<< dimGrid, dimBlock, sizeSmem >>>(switchParameters, m, n, mtile, ntile, lds);

#define workSizeSwitch_n(kernel, BLOCKSIZE_m, WORKSIZE_m, workSize_n) \
	switch (workSize_n)\
	{\
	case 1:\
		kernel<BLOCKSIZE_m, WORKSIZE_m, 1><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 2:\
		kernel<BLOCKSIZE_m, WORKSIZE_m, 2><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 3:\
		kernel<BLOCKSIZE_m, WORKSIZE_m, 3><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 4:\
		kernel<BLOCKSIZE_m, WORKSIZE_m, 4><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 5:\
		kernel<BLOCKSIZE_m, WORKSIZE_m, 5><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 6:\
		kernel<BLOCKSIZE_m, WORKSIZE_m, 6><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 7:\
		kernel<BLOCKSIZE_m, WORKSIZE_m, 7><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 8:\
		kernel<BLOCKSIZE_m, WORKSIZE_m, 8><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	default:\
		printf("\n workSizeSwitch_n : workSize not supported. [m = %i n = %i workSize_m = %i workSize_n = %i (workSize = %i)]\n", m, n, workSize_m, workSize_n, workSize_m * workSize_n); \
		exit(EXIT_FAILURE);\
	}

//
#define workSizeSwitch_n2(kernel, BLOCKSIZE_m, WORKSIZE_m, workSize_n, temp) \
	switch (workSize_n)\
	{\
	case 1:\
		kernel<BLOCKSIZE_m, WORKSIZE_m, 1><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile, temp); break; \
	case 2:\
		kernel<BLOCKSIZE_m, WORKSIZE_m, 2><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile, temp); break; \
	case 3:\
		kernel<BLOCKSIZE_m, WORKSIZE_m, 3><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile, temp); break; \
	case 4:\
		kernel<BLOCKSIZE_m, WORKSIZE_m, 4><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile, temp); break; \
	case 5:\
		kernel<BLOCKSIZE_m, WORKSIZE_m, 5><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile, temp); break; \
	case 6:\
		kernel<BLOCKSIZE_m, WORKSIZE_m, 6><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile, temp); break; \
	case 7:\
		kernel<BLOCKSIZE_m, WORKSIZE_m, 7><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile, temp); break; \
	case 8:\
		kernel<BLOCKSIZE_m, WORKSIZE_m, 8><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile, temp); break; \
	default:\
		printf("\n workSizeSwitch_n : workSize not supported. [m = %i n = %i workSize_m = %i workSize_n = %i (workSize = %i)]\n", m, n, workSize_m, workSize_n, workSize_m * workSize_n); \
		exit(EXIT_FAILURE);\
	}

#define workSizeSwitch_mn(kernel, BLOCKSIZE_m, workSize_m, workSize_n) \
	switch (workSize_m)\
	{\
	case 1:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 1, workSize_n); break; \
	case 2:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 2, workSize_n); break; \
	case 3:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 3, workSize_n); break; \
	case 4:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 4, workSize_n); break; \
	case 5:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 5, workSize_n); break; \
	case 6:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 6, workSize_n); break; \
	case 7:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 7, workSize_n); break; \
	case 8:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 8, workSize_n); break; \
	default:\
		printf("\n workSizeSwitch_m : workSize not supported. [m = %i n = %i workSize_m = %i workSize_n = %i (workSize = %i)]\n", m, n, workSize_m, workSize_n, workSize_m * workSize_n);\
		exit(EXIT_FAILURE);\
	}

//

#define workSizeSwitchLarge_mn(kernel, BLOCKSIZE_m, workSize_m, workSize_n) \
	switch (workSize_m)\
	{\
	case 1:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 1, workSize_n); break; \
	case 2:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 2, workSize_n); break; \
	case 3:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 3, workSize_n); break; \
	case 4:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 4, workSize_n); break; \
	case 5:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 5, workSize_n); break; \
	case 6:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 6, workSize_n); break; \
	case 7:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 7, workSize_n); break; \
	case 8:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 8, workSize_n); break; \
	case 9:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 9, workSize_n); break; \
	case 10:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 10, workSize_n); break; \
	case 11:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 11, workSize_n); break; \
	case 12:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 12, workSize_n); break; \
	case 13:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 13, workSize_n); break; \
	case 14:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 14, workSize_n); break; \
	case 15:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 15, workSize_n); break; \
	case 16:\
		workSizeSwitch_n(kernel, BLOCKSIZE_m, 16, workSize_n); break; \
	default:\
		printf("\n workSizeSwitchLarge_m : workSize not supported. [m = %i n = %i workSize_m = %i workSize_n = %i (workSize = %i)]\n", m, n, workSize_m, workSize_n, workSize_m * workSize_n);\
		exit(EXIT_FAILURE);\
	}

#define workSizeSwitchLarge_m(kernel, BLOCKSIZE_m, workSize_m, WORKSIZE_n) \
	switch (workSize_m)\
	{\
	case 1:\
		kernel<BLOCKSIZE_m, 1, WORKSIZE_n><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 2:\
		kernel<BLOCKSIZE_m, 2, WORKSIZE_n><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 3:\
		kernel<BLOCKSIZE_m, 3, WORKSIZE_n><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 4:\
		kernel<BLOCKSIZE_m, 4, WORKSIZE_n><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 5:\
		kernel<BLOCKSIZE_m, 5, WORKSIZE_n><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 6:\
		kernel<BLOCKSIZE_m, 6, WORKSIZE_n><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 7:\
		kernel<BLOCKSIZE_m, 7, WORKSIZE_n><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 8:\
		kernel<BLOCKSIZE_m, 8, WORKSIZE_n><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 9:\
		kernel<BLOCKSIZE_m, 9, WORKSIZE_n><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 10:\
		kernel<BLOCKSIZE_m, 10, WORKSIZE_n><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 11:\
		kernel<BLOCKSIZE_m, 11, WORKSIZE_n><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 12:\
		kernel<BLOCKSIZE_m, 12, WORKSIZE_n><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 13:\
		kernel<BLOCKSIZE_m, 13, WORKSIZE_n><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 14:\
		kernel<BLOCKSIZE_m, 14, WORKSIZE_n><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 15:\
		kernel<BLOCKSIZE_m, 15, WORKSIZE_n><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 16:\
		kernel<BLOCKSIZE_m, 16, WORKSIZE_n><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	default:\
		printf("\n workSizeSwitchLarge_m : workSize not supported. [m = %i n = %i workSize_m = %i workSize_n = %i (workSize = %i)]\n", m, n, workSize_m, WORKSIZE_n, workSize_m * WORKSIZE_n);\
		exit(EXIT_FAILURE);\
	}

#define workSizeSwitch_Large_n(kernel, BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, workSize_n) \
	switch (workSize_n)\
	{\
	case 1:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 1><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 2:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 2><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 3:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 3><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 4:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 4><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 5:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 5><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 6:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 6><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 7:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 7><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 8:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 8><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 9:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 9><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 10:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 10><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 11:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 11><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 12:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 12><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 13:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 13><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 14:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 14><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 15:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 15><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	case 16:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 16><<< dimGrid, dimBlock >>>(switchParameters, m, n, mtile, ntile); break; \
	default:\
		printf("\n workSizeSwitch_Large_n : workSize not supported. [m = %i n = %i workSize_m = %i workSize_n = %i (workSize = %i)]\n", m, n, workSize_m, workSize_n, workSize_m * workSize_n); \
		exit(EXIT_FAILURE);\
	}

#define workSizeSwitch_m_Large_n(kernel, BLOCKSIZE_m, BLOCKSIZE_n, workSize_m, workSize_n) \
	switch (workSize_m)\
	{\
	case 1:\
		workSizeSwitch_Large_n(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 1, workSize_n); break; \
	case 2:\
		workSizeSwitch_Large_n(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 2, workSize_n); break; \
	case 3:\
		workSizeSwitch_Large_n(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 3, workSize_n); break; \
	case 4:\
		workSizeSwitch_Large_n(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 4, workSize_n); break; \
	case 5:\
		workSizeSwitch_Large_n(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 5, workSize_n); break; \
	case 6:\
		workSizeSwitch_Large_n(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 6, workSize_n); break; \
	case 7:\
		workSizeSwitch_Large_n(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 7, workSize_n); break; \
	case 8:\
		workSizeSwitch_Large_n(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 8, workSize_n); break; \
	default:\
		printf("\n workSizeSwitch_m_Large_n : workSize not supported. [m = %i n = %i workSize_m = %i workSize_n = %i (workSize = %i)]\n", m, n, workSize_m, workSize_n, workSize_m * workSize_n);\
		exit(EXIT_FAILURE);\
	}

#define workSizeSwitch_Pow_n(kernel, BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, workSize_n) \
	switch (workSize_n)\
	{\
	case 1:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 1><<< dimGrid, dimBlock >>>(alpha, A_d, lda, x_d, beta, y_d, m, n, mtile, ntile); break; \
	case 2:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 2><<< dimGrid, dimBlock >>>(alpha, A_d, lda, x_d, beta, y_d, m, n, mtile, ntile); break; \
	case 4:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 4><<< dimGrid, dimBlock >>>(alpha, A_d, lda, x_d, beta, y_d, m, n, mtile, ntile); break; \
	case 8:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 8><<< dimGrid, dimBlock >>>(alpha, A_d, lda, x_d, beta, y_d, m, n, mtile, ntile); break; \
	case 16:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 16><<< dimGrid, dimBlock >>>(alpha, A_d, lda, x_d, beta, y_d, m, n, mtile, ntile); break; \
	case 32:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 32><<< dimGrid, dimBlock >>>(alpha, A_d, lda, x_d, beta, y_d, m, n, mtile, ntile); break; \
	case 64:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 64><<< dimGrid, dimBlock >>>(alpha, A_d, lda, x_d, beta, y_d, m, n, mtile, ntile); break; \
	case 128:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 128><<< dimGrid, dimBlock >>>(alpha, A_d, lda, x_d, beta, y_d, m, n, mtile, ntile); break; \
	default:\
		printf("\n workSizeSwitch_Pow_n : workSize not supported. [m = %i n = %i workSize_m = %i workSize_n = %i (workSize = %i)]\n", m, n, workSize_m, workSize_n, workSize_m * workSize_n); \
		exit(EXIT_FAILURE);\
	}

#define workSizeSwitch_m_Pow_n(kernel, BLOCKSIZE_m, BLOCKSIZE_n, workSize_m, workSize_n) \
	switch (workSize_m)\
	{\
	case 1:\
		workSizeSwitch_Pow_n(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 1, workSize_n); break; \
	case 2:\
		workSizeSwitch_Pow_n(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 2, workSize_n); break; \
	case 3:\
		workSizeSwitch_Pow_n(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 3, workSize_n); break; \
	case 4:\
		workSizeSwitch_Pow_n(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 4, workSize_n); break; \
	case 5:\
		workSizeSwitch_Pow_n(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 5, workSize_n); break; \
	case 6:\
		workSizeSwitch_Pow_n(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 6, workSize_n); break; \
	case 7:\
		workSizeSwitch_Pow_n(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 7, workSize_n); break; \
	case 8:\
		workSizeSwitch_Pow_n(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 8, workSize_n); break; \
	default:\
		printf("\n workSizeSwitch_m_Pow_n : workSize not supported. [m = %i n = %i workSize_m = %i workSize_n = %i (workSize = %i)]\n", m, n, workSize_m, workSize_n, workSize_m * workSize_n);\
		exit(EXIT_FAILURE);\
	}

#define blockSizeSwitch_m(kernel, blockSize_m, blockSize_n, workSize_m, workSize_n) \
	switch (blockSize_m)\
	{\
	case 1:\
		workSizeSwitch_Pow_n(kernel, 1, BLOCKSIZE / 1 + (BLOCKSIZE / 1 == 0), 1, workSize_n); break; \
	case 2:\
		workSizeSwitch_Pow_n(kernel, 2, BLOCKSIZE / 2 + (BLOCKSIZE / 2 == 0), 1, workSize_n); break; \
	case 4:\
		workSizeSwitch_Pow_n(kernel, 4, BLOCKSIZE / 4 + (BLOCKSIZE / 4 == 0), 1, workSize_n); break; \
	case 8:\
		workSizeSwitch_Pow_n(kernel, 8, BLOCKSIZE / 8 + (BLOCKSIZE / 8 == 0), 1, workSize_n); break; \
	case 16:\
		workSizeSwitch_Pow_n(kernel, 16, BLOCKSIZE / 16 + (BLOCKSIZE / 16 == 0), 1, workSize_n); break; \
	case 32:\
		workSizeSwitch_Pow_n(kernel, 32, BLOCKSIZE / 32 + (BLOCKSIZE / 32 == 0), 1, workSize_n); break; \
	case 64:\
		workSizeSwitch_Pow_n(kernel, 64, BLOCKSIZE / 64 + (BLOCKSIZE / 64 == 0), 1, workSize_n); break; \
	case 128:\
		workSizeSwitch_Pow_n(kernel, 128, BLOCKSIZE / 128 + (BLOCKSIZE / 128 == 0), 1, workSize_n); break; \
	case 256:\
		workSizeSwitch_Pow_n(kernel, 256, BLOCKSIZE / 256 + (BLOCKSIZE / 256 == 0), 1, workSize_n); break; \
	default:\
		printf("\n blockSizeSwitch_m : blocksize not supported. [blocksize_m = %i blocksize_n = %i (blocksize = %i)]\n", blockSize_m, blockSize_n, blockSize_m * blockSize_n); \
		exit(EXIT_FAILURE);\
	}

#define workSizeSwitch_Pow_n_sm(kernel, BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, workSize_n) \
	switch (workSize_n)\
	{\
	case 1:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 1><<< dimGrid, dimBlock, sizeSmem >>>(switchParameters, m, n, mtile, ntile, lds); break; \
	case 2:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 2><<< dimGrid, dimBlock, sizeSmem >>>(switchParameters, m, n, mtile, ntile, lds); break; \
	case 4:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 4><<< dimGrid, dimBlock, sizeSmem >>>(switchParameters, m, n, mtile, ntile, lds); break; \
	case 8:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 8><<< dimGrid, dimBlock, sizeSmem >>>(switchParameters, m, n, mtile, ntile, lds); break; \
	case 16:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 16><<< dimGrid, dimBlock, sizeSmem >>>(switchParameters, m, n, mtile, ntile, lds); break; \
	case 32:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 32><<< dimGrid, dimBlock, sizeSmem >>>(switchParameters, m, n, mtile, ntile, lds); break; \
	case 64:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 64><<< dimGrid, dimBlock, sizeSmem >>>(switchParameters, m, n, mtile, ntile, lds); break; \
	case 128:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m, 128><<< dimGrid, dimBlock, sizeSmem >>>(switchParameters, m, n, mtile, ntile, lds); break; \
	default:\
		printf("\n workSizeSwitch_Pow_n_sm : workSize not supported. [m = %i n = %i workSize_m = %i workSize_n = %i (workSize = %i)]\n", m, n, workSize_m, workSize_n, workSize_m * workSize_n); \
		exit(EXIT_FAILURE);\
	}

#define workSizeSwitch_m_Pow_n_sm(kernel, BLOCKSIZE_m, BLOCKSIZE_n, workSize_m, workSize_n) \
	switch (workSize_m)\
	{\
	case 1:\
		workSizeSwitch_Pow_n_sm(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 1, workSize_n); break; \
	case 2:\
		workSizeSwitch_Pow_n_sm(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 2, workSize_n); break; \
	case 3:\
		workSizeSwitch_Pow_n_sm(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 3, workSize_n); break; \
	case 4:\
		workSizeSwitch_Pow_n_sm(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 4, workSize_n); break; \
	case 5:\
		workSizeSwitch_Pow_n_sm(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 5, workSize_n); break; \
	case 6:\
		workSizeSwitch_Pow_n_sm(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 6, workSize_n); break; \
	case 7:\
		workSizeSwitch_Pow_n_sm(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 7, workSize_n); break; \
	case 8:\
		workSizeSwitch_Pow_n_sm(kernel, BLOCKSIZE_m, BLOCKSIZE_n, 8, workSize_n); break; \
	default:\
		printf("\n workSizeSwitch_m_Pow_n_sm : workSize not supported. [m = %i n = %i workSize_m = %i workSize_n = %i (workSize = %i)]\n", m, n, workSize_m, workSize_n, workSize_m * workSize_n);\
		exit(EXIT_FAILURE);\
	}

#define workSizeSwitch_m_sm(kernel, BLOCKSIZE_m, BLOCKSIZE_n, workSize_m, WORKSIZE_n) \
	switch (workSize_m)\
	{\
	case 1:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, 1, WORKSIZE_n><<< dimGrid, dimBlock, sizeSmem >>>(switchParameters, m, n, mtile, ntile, lds); break; \
	case 2:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, 2, WORKSIZE_n><<< dimGrid, dimBlock, sizeSmem >>>(switchParameters, m, n, mtile, ntile, lds); break; \
	case 3:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, 3, WORKSIZE_n><<< dimGrid, dimBlock, sizeSmem >>>(switchParameters, m, n, mtile, ntile, lds); break; \
	case 4:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, 4, WORKSIZE_n><<< dimGrid, dimBlock, sizeSmem >>>(switchParameters, m, n, mtile, ntile, lds); break; \
	case 5:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, 5, WORKSIZE_n><<< dimGrid, dimBlock, sizeSmem >>>(switchParameters, m, n, mtile, ntile, lds); break; \
	case 6:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, 6, WORKSIZE_n><<< dimGrid, dimBlock, sizeSmem >>>(switchParameters, m, n, mtile, ntile, lds); break; \
	case 7:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, 7, WORKSIZE_n><<< dimGrid, dimBlock, sizeSmem >>>(switchParameters, m, n, mtile, ntile, lds); break; \
	case 8:\
		kernel<BLOCKSIZE_m, BLOCKSIZE_n, 8, WORKSIZE_n><<< dimGrid, dimBlock, sizeSmem >>>(switchParameters, m, n, mtile, ntile, lds); break; \
	default:\
		printf("\n workSizeSwitch_m_sm : workSize not supported. [m = %i n = %i workSize_m = %i workSize_n = %i (workSize = %i)]\n", m, n, workSize_m, workSize_n, workSize_m * workSize_n); \
		exit(EXIT_FAILURE);\
	}

#define blockSizeSwitch_m_sm(kernel, blockSize_m, blockSize_n, workSize_m, workSize_n) \
	switch (blockSize_m)\
	{\
	case 1:\
		workSizeSwitch_m_sm(kernel, 1, BLOCKSIZE / 1 + (BLOCKSIZE / 1 == 0), workSize_m, workSize_n); break; \
	case 2:\
		workSizeSwitch_m_sm(kernel, 2, BLOCKSIZE / 2 + (BLOCKSIZE / 2 == 0), workSize_m, workSize_n); break; \
	case 4:\
		workSizeSwitch_m_sm(kernel, 4, BLOCKSIZE / 4 + (BLOCKSIZE / 4 == 0), workSize_m, workSize_n); break; \
	case 8:\
		workSizeSwitch_m_sm(kernel, 8, BLOCKSIZE / 8 + (BLOCKSIZE / 8 == 0), workSize_m, workSize_n); break; \
	case 16:\
		workSizeSwitch_m_sm(kernel, 16, BLOCKSIZE / 16 + (BLOCKSIZE / 16 == 0), workSize_m, workSize_n); break; \
	case 32:\
		workSizeSwitch_m_sm(kernel, 32, BLOCKSIZE / 32 + (BLOCKSIZE / 32 == 0), workSize_m, workSize_n); break; \
	case 64:\
		workSizeSwitch_m_sm(kernel, 64, BLOCKSIZE / 64 + (BLOCKSIZE / 64 == 0), workSize_m, workSize_n); break; \
	case 128:\
		workSizeSwitch_m_sm(kernel, 128, BLOCKSIZE / 128 + (BLOCKSIZE / 128 == 0), workSize_m, workSize_n); break; \
	case 256:\
		workSizeSwitch_m_sm(kernel, 256, BLOCKSIZE / 256 + (BLOCKSIZE / 256 == 0), workSize_m, workSize_n); break; \
	default:\
		printf("\n blockSizeSwitch_m_sm : blocksize not supported. [blocksize_m = %i blocksize_n = %i (blocksize = %i)]\n", blockSize_m, blockSize_n, blockSize_m * blockSize_n); \
		exit(EXIT_FAILURE);\
	}

