// 
// Generic vector scale and add kernel.
//
// - Simple design for all capabilities
// - Handled by splitting matrix as
// --------------------------
// |                 |      |
// |                 |      |
// |                 |      |
// | left tiled part | rest | 
// |                 |      |
// |                 |      |
// |                 |      |
// --------------------------
// - Kernels are generic and work for all cases
//

// Version 1: copy as floats
// -----------------------------------------
// Threads per block    : BLOCKSIZE_n
// Work per thread      : WORKSIZE_n
// Shared mem per block : 0
//
template <int BLOCKSIZE_n, int WORKSIZE_m, int WORKSIZE_n>
__launch_bounds__(BLOCKSIZE_n, MIN_BLOCKS_PER_MP)
__global__ void srot_kernel1(float* x, float* y, const float c, const float s, int n, int ntile)
{
	const int tid = threadIdx.x;
	const int idx = blockIdx.x * BLOCKSIZE_n * WORKSIZE_n;
	x += idx + tid;
	y += idx + tid;
	if (idx != ntile)
	{
        #pragma unroll
		for (int i = 0; i < WORKSIZE_n; i++ )
		{
			float vx = x[0], vy = y[0]; 
			x[0] = c * vx + s * vy;
			y[0] = c * vy - s * vx;
			x += BLOCKSIZE_n; y += BLOCKSIZE_n;
		}
	}
	else
	{
        #pragma unroll 4
		for (int i = 0; i < n - idx - tid; i += BLOCKSIZE_n)
		{
			float vx = x[i], vy = y[i]; 
			x[i] = c * vx + s * vy;
			y[i] = c * vy - s * vx;
		}
	}
}
// Version 2: copy as doubles (requires WORKSIZE_n & 1 == 0)
// -----------------------------------------
// Threads per block    : BLOCKSIZE_n
// Work per thread      : WORKSIZE_n / 2
// Shared mem per block : 0
//
template <int BLOCKSIZE_n, int WORKSIZE_m, int WORKSIZE_n>
__launch_bounds__(BLOCKSIZE_n, MIN_BLOCKS_PER_MP)
__global__ void srot_kernel2(float* x, float* y, const float c, const float s, int n, int ntile)
{
/*
	const int tid = threadIdx.x;
	const int idx = blockIdx.x * BLOCKSIZE_n * WORKSIZE_n;
	x += idx;
	y += idx;
	const double *xd = (double *) x + tid;
	double *yd = (double *) y + tid;
	__shared__ double dx_sh[BLOCKSIZE_n];
	__shared__ double dy_sh[BLOCKSIZE_n];
	double *dx_s = dx_sh + tid;
	double *dy_s = dy_sh + tid;
	float *x_s = (float *) dx_s;
	float *y_s = (float *) dy_s;
	if (idx != ntile)
	{
        #pragma unroll
		for (int i = 0; i < (WORKSIZE_n / 2); i++ )
		{
			dx_s[0] = xd[0]; dy_s[0] = yd[0]; 
			y_s[0] += alpha * x_s[0]; y_s[1] += alpha * x_s[1]; 
			yd[0] = *dy_s; xd += BLOCKSIZE_n; yd += BLOCKSIZE_n;
		}
	}
	else
	{
        #pragma unroll 4
		for (int i = 0; i < ((n - idx) >> 1) - tid; i += BLOCKSIZE_n)
		{
			dx_s[0] = xd[i]; 
			dy_s[0] = yd[i]; 
			y_s[0] += alpha * x_s[0]; 
			y_s[1] += alpha * x_s[1]; 
			yd[i] = *dy_s;
		}
	}
	if (blockIdx.x == 0 && tid == 0 && n & 1) y[n - 1] += alpha * x[n - 1];
*/
}

